%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%                                                                  %%%%%
%%%%    IEEE PES Power Grid Library - Optimal Power Flow - v19.05     %%%%%
%%%%          (https://github.com/power-grid-lib/pglib-opf)           %%%%%
%%%%               Benchmark Group - Typical Operations               %%%%%
%%%%                         10 - May - 2019                          %%%%%
%%%%                                                                  %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Power flow data for the IEEE RELIABILITY TEST SYSTEM 1979.
%
%   IEEE Reliability Test System Task Force of the Applications of
%   Probability Methods Subcommittee, "IEEE reliability test system,"
%   IEEE Transactions on Power Apparatus and Systems, Vol. 98, No. 6,
%   Nov./Dec. 1979, pp. 2047-2054.
%
%   Cost data is from Web site run by Georgia Tech Power Systems Control
%   and Automation Laboratory:
%       http://pscal.ece.gatech.edu/testsys/index.html
%
%   Matpower case file data provided by Bruce Wollenberg.
%
%   Copyright (c) 1979 The Institute of Electrical and Electronics Engineers (IEEE)
%   Licensed under the Creative Commons Attribution 4.0
%   International license, http://creativecommons.org/licenses/by/4.0/
%
%   Contact M.E. Brennan (me.brennan@ieee.org) for inquries on further reuse of
%   this dataset.
%		transmission expansion data from: Power system transmission network expansion planning using AC model by M.J. Rider, A.V. Garcia and R. Romero

function mpc = pglib_opf_case24_ieee_rts
mpc.version = '2';
mpc.baseMVA = 100.0;

%% area data
%	area	refbus
mpc.areas = [
	1	 1;
	2	 3;
	3	 8;
	4	 6;
];

%% bus data
%	bus_i	type	Pd	Qd	Gs	Bs	area	Vm	Va	baseKV	zone	Vmax	Vmin
mpc.bus = [
	1	 3	 324		 66 	 0.0	 0.0	 1	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	2	 2	 291		 60 	 0.0	 0.0	 1	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	3	 1	 540		 111	 0.0	 0.0	 1	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	4	 1	 222		 45 	 0.0	 0.0	 1	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	5	 1	 213		 42 	 0.0	 0.0	 1	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	6	 1	 408		 84 	 0.0	 -100.0	 2	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	7	 2	 375		 75 	 0.0	 0.0	 2	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	8	 1	 513		 105 	 0.0	 0.0	 2	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	9	 1	 525		 108  	 0.0	 0.0	 1	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	10	 1	 585		 120  	 0.0	 0.0	 2	    1.00000	    0.00000	 138.0	 1	    1.1    0.9;
	11	 1	 0			 0 	 	 0.0	 0.0	 3	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	12	 1	 0			 0 	 	 0.0	 0.0	 3	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	13	 2	 795		 162     0.0	 0.0	 3	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	14	 2	 582		 117     0.0	 0.0	 3	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	15	 2	 951		 192     0.0	 0.0	 4	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	16	 2	 300		 60 	 0.0	 0.0	 4	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	17	 1	 0   		 0 	 	 0.0	 0.0	 4	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	18	 2	 999 		 204	 0.0	 0.0	 4	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	19	 1	 543 		 111	 0.0	 0.0	 3	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	20	 1	 384 	 	 78 	 0.0	 0.0	 3	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	21	 2	 0			 0 	 	 0.0	 0.0	 4	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	22	 2	 0.0	 	 0.0	 0.0	 0.0	 4	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	23	 2	 0.0	 	 0.0	 0.0	 0.0	 3	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
	24	 1	 0.0	 	 0.0	 0.0	 0.0	 4	    1.00000	    0.00000	 230.0	 1	    1.1    0.9;
];

%% generator data
%	bus	Pg	Qg	Qmax	Qmin	Vg	mBase	status	Pmax	Pmin
mpc.gen = [
	1	 45.6	 2.5	 240.0	 -150.0	 1.0	 100.0	 1	 576	 0;
	2	 18.0	 5.0	 240.0	 -150.0	 1.0	 100.0	 1	 576	 0;
	6	 18.0	 5.0	 0	 	 -300.0	 1.0	 100.0	 1	 0	 	 0;
	7	 62.5	 30.0	 540.0	 0.0	 1.0	 100.0	 1	 900.0	 0;
	13	 133.0	 40.0	 720.0	 0.0	 1.0	 100.0	 1	 1773.0	 0;
	14	 0.0	 75.0	 600.0	 -150.0	 1.0	 100.0	 1	 0.0	 0;
	15	 7.2	 3.0	 330.0	 -150.0	 1.0	 100.0	 1	 645.0	 0;
	16	 104.65	 15.0	 240.0	 -150.0	 1.0	 100.0	 1	 465.0	 0;
	18	 250.0	 75.0	 600.0	 -150.0	 1.0	 100.0	 1	 1200.0	 0;
	21	 250.0	 75.0	 600.0	 -150.0	 1.0	 100.0	 1	 1200.0	 0;
	22	 30.0	 3.0	 288.0	 -180.0	 1.0	 100.0	 1	 900.0	 0;
	23	 104.65	 15.0	 930.0	 -375.0	 1.0	 100.0	 1	 1980.0	 0;
];

%% generator cost data
%	2	startup	shutdown	n	c(n-1)	...	c0
mpc.gencost = [
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
	2	 1500.0	 0.0	 3	   0.000000	 0.00000000	 0.00000000;
];

%% branch data
%	fbus	tbus	r	x	b	rateA	rateB	rateC	ratio	angle	status	angmin	angmax
mpc.branch = [
1	 2	 0.0026	 0.0139	 0.4611	 200.0	 193.0	 200.0	 0.0	 0.0	 1	 -30.0	 30.0;
1	 3	 0.0546	 0.2112	 0.0572	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
1	 5	 0.0218	 0.0845	 0.0229	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
2	 4	 0.0328	 0.1267	 0.0343	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
2	 6	 0.0497	 0.192	 0.052	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
3	 9	 0.0308	 0.119	 0.0322	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
3	 24	 0.0023	 0.0839	 0.0	 600.0	 510.0	 600.0	 1.03	 0.0	 1	 -30.0	 30.0;
4	 9	 0.0268	 0.1037	 0.0281	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
5	 10	 0.0228	 0.0883	 0.0239	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
6	 10	 0.0139	 0.0605	 2.459	 200.0	 193.0	 200.0	 0.0	 0.0	 1	 -30.0	 30.0;
7	 8	 0.0159	 0.0614	 0.0166	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
8	 9	 0.0427	 0.1651	 0.0447	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
8	 10	 0.0427	 0.1651	 0.0447	 220.0	 208.0	 220.0	 0.0	 0.0	 1	 -30.0	 30.0;
9	 11	 0.0023	 0.0839	 0.0	 600.0	 510.0	 600.0	 1.03	 0.0	 1	 -30.0	 30.0;
9	 12	 0.0023	 0.0839	 0.0	 600.0	 510.0	 600.0	 1.03	 0.0	 1	 -30.0	 30.0;
10	 11	 0.0023	 0.0839	 0.0	 600.0	 510.0	 600.0	 1.02	 0.0	 1	 -30.0	 30.0;
10	 12	 0.0023	 0.0839	 0.0	 600.0	 510.0	 600.0	 1.02	 0.0	 1	 -30.0	 30.0;
11	 13	 0.0061	 0.0476	 0.0999	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
11	 14	 0.0054	 0.0418	 0.0879	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
12	 13	 0.0061	 0.0476	 0.0999	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
12	 23	 0.0124	 0.0966	 0.203	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
13	 23	 0.0111	 0.0865	 0.1818	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
14	 16	 0.005	 0.0389	 0.0818	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
15	 16	 0.0022	 0.0173	 0.0364	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
15	 21	 0.0063	 0.049	 0.103	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
15	 21	 0.0063	 0.049	 0.103	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
15	 24	 0.0067	 0.0519	 0.1091	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
16	 17	 0.0033	 0.0259	 0.0545	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
16	 19	 0.003	 0.0231	 0.0485	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
17	 18	 0.0018	 0.0144	 0.0303	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
17	 22	 0.0135	 0.1053	 0.2212	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
18	 21	 0.0033	 0.0259	 0.0545	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
18	 21	 0.0033	 0.0259	 0.0545	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
19	 20	 0.0051	 0.0396	 0.0833	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
19	 20	 0.0051	 0.0396	 0.0833	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
20	 23	 0.0028	 0.0216	 0.0455	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
20	 23	 0.0028	 0.0216	 0.0455	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
21	 22	 0.0087	 0.0678	 0.1424	 625.0	 600.0	 625.0	 0.0	 0.0	 1	 -30.0	 30.0;
];


%% candidate dc bus data
%column_names%   busdc_i grid    Pdc     Vdc     basekVdc    Vdcmax  Vdcmin  Cdc
mpc.busdc_ne = [
1              1       0       1       345       1.1     0.9    0;
2              1       0       1       345       1.1     0.9    0;
3              1       0       1       345       1.1     0.9    0;
4              1       0       1       345       1.1     0.9    0;
5              1       0       1       345       1.1     0.9    0;
6              1       0       1       345       1.1     0.9    0;
7              1       0       1       345       1.1     0.9    0;
8              1       0       1       345       1.1     0.9    0;
9              1       0       1       345       1.1     0.9    0;
10              1       0       1       345       1.1     0.9    0;
11              1       0       1       345       1.1     0.9    0;
12              1       0       1       345       1.1     0.9    0;
13              1       0       1       345       1.1     0.9    0;
14              1       0       1       345       1.1     0.9    0;
15              1       0       1       345       1.1     0.9    0;
16              1       0       1       345       1.1     0.9    0;
17              1       0       1       345       1.1     0.9    0;
18              1       0       1       345       1.1     0.9    0;
19              1       0       1       345       1.1     0.9    0;
20              1       0       1       345       1.1     0.9    0;
21              1       0       1       345       1.1     0.9    0;
22              1       0       1       345       1.1     0.9    0;
23              1       0       1       345       1.1     0.9    0;
24              1       0       1       345       1.1     0.9    0;
];


%% candidate branches
%column_names%   fbusdc  tbusdc  r      l        c   rateA   rateB   rateC status cost
mpc.branchdc_ne = [
1	 2	 0.0026	 0.00	 0.00  200.0		 0.0	 0.0	 1.0	3;
1	 3	 0.0546	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	55;
1	 5	 0.0218	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	22;
1	 8   0.0348  0.00  0.00  220.0     0.0   0.0   1.0  35;
2	 4	 0.0328	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	33;
2	 6	 0.0497	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	50;
2	 8	 0.0328	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	33;
3	 9	 0.0308	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	31;
3	 24	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
4	 9	 0.0268	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	27;
5	 10	 0.0228	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	23;
6	 7	 0.0497	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	50;
6	 10	 0.0139	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	16;
7	 8	 0.0159	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	16;
8	 9	 0.0427	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	43;
8	 10	 0.0427	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	43;
9	 11	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
9	 12	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
10	 11	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
10	 12	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
11	 13	 0.0061	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	66;
11	 14	 0.0054	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	58;
12	 13	 0.0061	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	66;
12	 23	 0.0124	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	134;
13	 14	 0.0057	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	62;
13	 23	 0.0111	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	120;
14	 16	 0.005	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	54;
15	 16	 0.0022	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	24;
14	 23	 0.008	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	86;
15	 21	 0.0063	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	68;
15	 24	 0.0067	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	72;
16	 17	 0.0033	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	36;
16	 19	 0.003	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	32;
16	 23	 0.0105	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	114;
17	 18	 0.0018	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	20;
17	 22	 0.0135	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	146;
18	 21	 0.0033	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	36;
19	 20	 0.0051	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	55;
19	 23	 0.0078	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	84;
20	 23	 0.0028	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	30;
21	 22	 0.0087	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	94;

1	 2	 0.0026	 0.00	 0.00  200.0		 0.0	 0.0	 1.0	3;
1	 3	 0.0546	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	55;
1	 5	 0.0218	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	22;
1	 8   0.0348  0.00  0.00  220.0     0.0   0.0   1.0  35;
2	 4	 0.0328	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	33;
2	 6	 0.0497	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	50;
2	 8	 0.0328	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	33;
3	 9	 0.0308	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	31;
3	 24	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
4	 9	 0.0268	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	27;
5	 10	 0.0228	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	23;
6	 7	 0.0497	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	50;
6	 10	 0.0139	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	16;
7	 8	 0.0159	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	16;
8	 9	 0.0427	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	43;
8	 10	 0.0427	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	43;
9	 11	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
9	 12	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
10	 11	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
10	 12	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
11	 13	 0.0061	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	66;
11	 14	 0.0054	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	58;
12	 13	 0.0061	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	66;
12	 23	 0.0124	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	134;
13	 14	 0.0057	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	62;
13	 23	 0.0111	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	120;
14	 16	 0.005	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	54;
15	 16	 0.0022	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	24;
14	 23	 0.008	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	86;
15	 21	 0.0063	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	68;
15	 24	 0.0067	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	72;
16	 17	 0.0033	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	36;
16	 19	 0.003	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	32;
16	 23	 0.0105	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	114;
17	 18	 0.0018	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	20;
17	 22	 0.0135	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	146;
18	 21	 0.0033	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	36;
19	 20	 0.0051	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	55;
19	 23	 0.0078	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	84;
20	 23	 0.0028	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	30;
21	 22	 0.0087	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	94;

1	 2	 0.0026	 0.00	 0.00  200.0		 0.0	 0.0	 1.0	3;
1	 3	 0.0546	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	55;
1	 5	 0.0218	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	22;
1	 8   0.0348  0.00  0.00  220.0     0.0   0.0   1.0  35;
2	 4	 0.0328	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	33;
2	 6	 0.0497	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	50;
2	 8	 0.0328	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	33;
3	 9	 0.0308	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	31;
3	 24	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
4	 9	 0.0268	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	27;
5	 10	 0.0228	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	23;
6	 7	 0.0497	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	50;
6	 10	 0.0139	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	16;
7	 8	 0.0159	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	16;
8	 9	 0.0427	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	43;
8	 10	 0.0427	 0.00	 0.00  220.0		 0.0	 0.0	 1.0	43;
9	 11	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
9	 12	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
10	 11	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
10	 12	 0.0023	 0.00	 0.00  600.0		 0.0	 0.0	 1.0	50;
11	 13	 0.0061	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	66;
11	 14	 0.0054	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	58;
12	 13	 0.0061	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	66;
12	 23	 0.0124	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	134;
13	 14	 0.0057	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	62;
13	 23	 0.0111	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	120;
14	 16	 0.005	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	54;
15	 16	 0.0022	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	24;
14	 23	 0.008	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	86;
15	 21	 0.0063	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	68;
15	 24	 0.0067	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	72;
16	 17	 0.0033	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	36;
16	 19	 0.003	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	32;
16	 23	 0.0105	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	114;
17	 18	 0.0018	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	20;
17	 22	 0.0135	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	146;
18	 21	 0.0033	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	36;
19	 20	 0.0051	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	55;
19	 23	 0.0078	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	84;
20	 23	 0.0028	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	30;
21	 22	 0.0087	 0.00	 0.00  625.0		 0.0	 0.0	 1.0	94;
];

%% candidate converters
%column_names%   busdc_i busac_i type_dc type_ac P_g   Q_g  islcc  Vtar rtf xtf  transformer tm   bf filter    rc      xc  reactor   basekVac Vmmax   Vmmin   Imax    status   LossA LossB  LossCrec LossCinv  droop Pdcset    Vdcset  dVdcset Pacmax Pacmin Qacmax Qacmin cost
	mpc.convdc_ne = [
1       1   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1    1.1033 0.887  2.885    2.885  				0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
2       2   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1    1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
3       3   2       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1    1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0        1000 -1000 500 -500 3;
4       4   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
5       5   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
6       6   2       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0      1000 -1000 500 -500 3;
7       7   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1    1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
8       8   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0      1000 -1000 500 -500 3;
9       9   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
10      10   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
11      11   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
12      12   2       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
13      13   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0      1000 -1000 500 -500 3;
14      14   2       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
15      15   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885  			0.0050    -52.7   1.0079  0     	 1000 -1000 500 -500 3;
16      16   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1    1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
17      17   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
18      18   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
19      19   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
20      20   2       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0      1000 -1000 500 -500 3;
21      21   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1    1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
22      22   1       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885         0.0050    -52.7   1.0079   0      1000 -1000 500 -500 3;
23      23   2       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
24      24   2       1       -360    -1.66   		0 1.0          0.0001  0.001 1 1 0.001 1 0.0001   0.001 1     345         1.1     0.9     100     1     1.1033 0.887  2.885    2.885        0.0050    -52.7   1.0079   0       1000 -1000 500 -500 3;
];
